package fr.iut.pm.movieapplication.ui.fragments

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import fr.iut.pm.movieapplication.R
import fr.iut.pm.movieapplication.model.media.MediaResult
import fr.iut.pm.movieapplication.model.media.movie.Movie
import fr.iut.pm.movieapplication.model.media.movie.MovieDetails
import fr.iut.pm.movieapplication.repository.MovieRepository
import fr.iut.pm.movieapplication.ui.activity.MainActivity
import fr.iut.pm.movieapplication.ui.adapter.HomeItemAdapter
import fr.iut.pm.movieapplication.ui.adapter.MovieAdapter
import fr.iut.pm.movieapplication.ui.viewmodel.MoviesVM
import fr.iut.pm.movieapplication.ui.viewmodel.MoviesVMFactory
import fr.iut.pm.movieapplication.utils.Constants.Companion.PAGE_SIZE

class MoviesFragment(
    private val context : MainActivity
) : Fragment() {

    private var isLoading = false
    private var isLastPage = false
    private var currentPage = 1
    private var currentList : MutableList<Movie> = mutableListOf()

    private val moviesVM: MoviesVM by viewModels{ MoviesVMFactory(repository = MovieRepository())}
    lateinit var moviesRecyclerView : RecyclerView
    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        val view = inflater.inflate(R.layout.fragment_movies, container, false)

        // Get the RecyclerView
        moviesRecyclerView = view.findViewById<RecyclerView>(R.id.movies_item_recycler_view)

        // Initialized the data inside our RecyclerView
        context.movieRepository.getPopularMovies { listMovies ->
            currentList.addAll(0,listMovies)
            moviesRecyclerView.adapter = MovieAdapter(context, R.layout.item_vertical_fragment, currentList)
            moviesRecyclerView.layoutManager = GridLayoutManager (context, 3)
        }

        // Create the ScrollListener
        val scrollListener = object : RecyclerView.OnScrollListener() {
            override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                super.onScrolled(recyclerView, dx, dy)

                val layoutManager = moviesRecyclerView.layoutManager as GridLayoutManager
                val visibleItemCount = layoutManager.childCount
                val totalItemCount = layoutManager.itemCount
                val firstVisibleItemPosition = layoutManager.findFirstVisibleItemPosition()

                // If we are not already loading data and it's not the last page
                if(!isLoading && !isLastPage) {
                    if(visibleItemCount + firstVisibleItemPosition >= totalItemCount
                        && firstVisibleItemPosition >= 0
                        && totalItemCount >= PAGE_SIZE
                    ) {
                        loadMoreMovies()
                    }
                }
            }
        }

        // Add the ScrollLister created before to our RecyclerView
        moviesRecyclerView.addOnScrollListener(scrollListener)

        return view
    }

    /**
     * Method to load data when the user reaches the bottom of the view
     */
    private fun loadMoreMovies() {
        isLoading = true
        currentPage += 1

        if(currentPage == 1000) isLastPage = true

        val start = currentList.size
        context.movieRepository.getPopularMovies(currentPage) { listMovies ->

            currentList.addAll(start, listMovies)
            moviesRecyclerView.adapter?.notifyItemRangeChanged(start, listMovies.size)

        }

        isLoading = false
    }
}